package com.allyourcode.p03_03_13;

/* Don't forget:
   Paste your weather underground
   key id in the URL below.
 */

import android.app.Service;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.widget.Toast;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import java.io.IOException;
import java.net.URL;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

public class MyWeatherService extends Service {

  Messenger messengerToClient = null;

  MyIncomingHandler myIncomingHandler =
      new MyIncomingHandler();
  Messenger messengerToService =
      new Messenger(myIncomingHandler);

  @Override
  public IBinder onBind(Intent intent) {
    doToast(R.string.service_bound);
    return messengerToService.getBinder();
  }

  class MyIncomingHandler extends Handler {

    @Override
    public void handleMessage(Message incomingMessage) {
      messengerToClient = incomingMessage.replyTo;
      new MyAsyncTask().execute(incomingMessage.getData().getString("location"));
    }
  }

  @Override
  public boolean onUnbind(Intent intent) {
    doToast(R.string.service_stopped_itself);
    stopSelf();
    return false;
  }

  @Override
  public void onDestroy() {
    myIncomingHandler = null;
    doToast(R.string.service_destroyed);
  }

  void doToast(int resource) {
    Toast.makeText(this, resource, Toast.LENGTH_SHORT).show();
  }

  class MyAsyncTask extends AsyncTask<String, Void, String> {

    @Override
    protected String doInBackground(String... locations) {
      return getWeatherFromInternet(locations[0]);
    }

    @Override
    protected void onPostExecute(String actualWeatherString) {
      sendWeatherToClient(actualWeatherString);
      doToast(R.string.message_handled);
    }

  }

  private static final String WEATHER_UNDERGROUND_URL =
     "http://api.wunderground.com/" +
        "api/your_key_id/conditions/q/";
  private static final String CONDITION = "weather";
  private static final String TEMP_F = "temp_f";

  String getWeatherFromInternet(String location) {
    Weather weather = null;
    String weatherString;

    URL url;

    if (location != null && !location.equals("")) {
      try {
        url = new URL
          (WEATHER_UNDERGROUND_URL + location + ".xml");

        SAXParser parser =
           SAXParserFactory.newInstance().newSAXParser();
        XMLReader reader = parser.getXMLReader();
        MySaxHandler saxHandler = new MySaxHandler();
        reader.setContentHandler(saxHandler);

        reader.parse(new InputSource(url.openStream()));

        weather = saxHandler.getWeather();


      } catch (SAXException |
               ParserConfigurationException |
               IOException e) {
        e.printStackTrace();
      }

      weatherString = weather.temperature
         + (char) 0x00B0 + "F " + weather.condition;
    } else {
      weatherString = "It's dark at night.";
    }
    return weatherString;
  }

  void sendWeatherToClient(String actualWeatherString) {
    Bundle reply = new Bundle();
    reply.putString("weather", actualWeatherString);
    Message replyMessage = Message.obtain();
    replyMessage.setData(reply);
    try {
      messengerToClient.send(replyMessage);
    } catch (RemoteException e) {
      e.printStackTrace();
    }
  }

}

